%% 1. ConfiguraCamera.m
% Faz a configurao da webcam para a captura de um vdeo. Retirado
% diretamente do captulo trs do documento *Mtodos de Aquisio de
% vdeo*, o indicador fsico das atividades 1 e 2 da primeira meta do
% projeto final *Transmisso de Vdeo para uma unidade mvel via EDGE*.

%% 1.1 - Configurando o hardware para a aquisio
%
% Iniciaremos, procurando as informaes de adaptadores de vdeo instalados
% no computador:
sistema = imaqhwinfo();
display('---------- Adaptadores Disponveis:')
adaptadores = sistema.InstalledAdaptors

%%
% Obteremos agora as informaes dos adaptadores disponveis. Estas
% informaes esto acessveis, passando  funo |_imaqhwinfo()_| o nome
% do adaptador.
%
% A sada apresenta as seguintes informaes:
%
% * *AdaptorDllName:* Caminho do arquivo DLL do adaptador
% * *AdaptorDllVersion:* Verso do DLL
% * *AdaptorName:* Nome do adaptador
% * *DeviceIDs:* Identificao dos _devices_ acessveis atravs do
% adaptador.  pelo _device_ que acessaremos a cmera.
% * *DeviceInfo:* Informaes sobre determinado device
display('---------- Informaes dos adaptadores disponveis:')
adaptador(1) = imaqhwinfo('coreco')
adaptador(2) = imaqhwinfo('winvideo')
%%
% Neste caso, estamos exibindo as informaes destes dois adaptadores.
% Podem existir computadores com mais ou menos. Deve-se observar ento a
% varivel |adaptadores|.
%
% Note que o campo *DeviceIDs* do adaptador _coreco_  uma clula de
% tamanho 1x0, ou seja, no existem _devices_ acessveis por este
% adaptador. Na maioria dos casos, o nico adaptador com _devices_
% instalados ser o _winvideo_.
%
% Visualizaremos as informaes do _device_. Para isto, passamos  funo
% |_imaqhwinfo()_|, alm do nome do adaptador, o _ID_ do _device_
% instalado. A sada apresenta entre outras, as seguintes informaes:
%
% * *DefaultFormat:* Formato de vdeo padro
% * *DeviceName:* Nome do device
% * *DeviceID:* Identidade do device
% * *SupportedFormats:* Formatos suportados pelo device
display('---------- Informaes do Device:')
DeviceInfo = imaqhwinfo('winvideo',1)

%% 
% Agora, obteremos os formatos suportados pelo device. Basta ento
% referenciar o campo _SupportedFormats_ da estrutura que contm as
% informaes do device.
display('---------- Formatos suportados pelo Device:')
formatos = DeviceInfo.SupportedFormats

%% 1.2 - Criando um objeto VideoInput
%
% A interface entre a cmera e o *MatLab*  feita por um objeto
% _videoinput_. Todas as operaes com o vdeo ser realizada atravs dele.
%
% Primeiramente, criaremos um objeto _videoinput_. Usaremos a funo
% |_videoinput()_|, que tem a seguinte sintaxe:
%
% |videoinput(<nome_do_adaptador>, <ID_do_device>, <formato_do_video>)|
%
% A sada  um objeto _videoinput_
display('---------- Criando um objeto videoinput:')
cores = uint16(24);
altura = uint16(240);
largura = uint16(320);
objeto = videoinput('winvideo',1,...
    ['RGB' num2str(cores) '_' num2str(largura) 'x' num2str(altura)])

%% 1.3 - Configurando os parmetros da captura
%
% Usa-se a funo |get(<objeto>)| para obter propriedades do objeto
% _videoinput_.
display('---------- Propriedades default do objeto:')
get(objeto)

%%
% * *Configurando o nmero de frames capturados pela webcam*
%
% A seguir, configuraremos o nmero de _frames_ capturados pela cmera a
% cada leitura.
%
% Para isto, usamos a funo |_set()_|, que possui a seguinte sintaxe:
%
% |set(<objeto>, <'Parmetro'>, <valor>)|
%
% Caso o |<valor>| no seja informado, a funo retornar todos os valores
% suportados pelo |<'Parmetro'>| do |<objeto>|.
display('---------- Configurando o nmero de quadros capturados a cada leitura')
set(objeto,'FramesPerTrigger',1);

%%
% * *Configurando o nmero de leituras realizadas*
%
% Podemos configurar tambm quantas leituras faremos da cmera. Este
% parmetro multiplicado pelo anterior indica quantos _frames_ sero
% capturados no total.
display('---------- Configurando o nmero de leituras realizadas')
set(objeto,'TriggerRepeat',inf);

%%
% * *Configurando o tipo de gatilho utilizado*
%
% O "gatilho" consiste no sinal recebido pelo *MatLab* que indica o incio
% da captura dos quadros. So trs os tipos de gatilhos suportados:
%
% *Imediato (_immediate_)* - indica ao *MatLab* para iniciar a captura assim
% que a funo |_start(objeto)_| for chamada. Esta funo inicializa as
% instncias e os temporizadores do objeto. Caso o objeto esteja
% configurado com um _TriggerRepeat_ maior que um, a aquisio ser feita
% continuamente, at o total de _frames_ ser capturado.
%
% *Manual (_manual_)* - indica que o sinal de incio da captura ser dado
% pelo script. Assim, o |_start(objeto)_| ser dado mas nenhum _frame_ ser
% capturado. Somente com a chamada da funo |_trigger(objeto)_| que a
% captura ser realizada. 
%
% A funo |_trigger(objeto)_| copia os _frames_ atuais da cmera para a
% memria do *MatLab*. O nmero de _frames_ adquiridos  indicado pelo
% parmetro _FramesPerTrigger_, configurado anteriormente. Caso o objeto
% esteja configurado com um _TriggerRepeat_ igual  um, um novo gatilho
% poder ser dado somente aps o reincio do objeto, o que apaga todos os
% dados da memria. Caso o objeto esteja configurado com um _TriggerRepeat_
% maior que um, novos gatilhos podero ser dados sem um novo incio do
% objeto, capturando sempre o nmero de _frames_ indicados pela propriedade
% _FramesPerTrigger_.
%
% *Por Hardware (_hardware_)* - o sinal de gatilho ser dado ao *MatLab*
% por um agente externo, como um sensor ou um boto. Toda a configurao de
% aquisio de frames ocorre de forma semelhante  do _trigger_ manual.
%
% A seguir, verificaremos os tipos de gatilho suportados pelo objeto. Para
% isto, usamos a funo |_triggerinfo(objeto)_|:
display('---------- Tipos de gatilhos suportados pelo objeto:')
triggerinfo(objeto)

%%
% Neste exemplo, setaremos o gatilho para *Manual*. Usamos ento a funo
% |_triggerconfig_| que tem a seguinte sintaxe:
%
% |triggerconfig(<objeto>,<valor>)|
display('---------- Configurando o gatilho como manual...')
triggerconfig(objeto,'Manual')

%%
% * *Configurando a taxa de frames por segundo do _device_*
%
% A seguir, configuraremos a taxa de _frames_ por segundo utilizada. Este 
% um parmetro especfico do _device_, que  configurado da seguinte forma:
display('---------- Configurando a taxa de frames por segundo...')
src = getselectedsource(objeto)
frameRates = set(src,'FrameRate')
src.FrameRate = frameRates{5}